@@ -49,7 +49,7 @@ class SortImports(object):
 
     def __init__(self, file_path=None, file_contents=None, file_=None, write_to_stdout=False, check=False,
                  show_diff=False, settings_path=None, ask_to_apply=False, run_path='', check_skip=True,
-                 **setting_overrides):
+                 extension=None, **setting_overrides):
         if not settings_path and file_path:
             settings_path = os.path.dirname(os.path.abspath(file_path))
         settings_path = settings_path or os.getcwd()
@@ -181,6 +181,11 @@ class SortImports(object):
                 self.in_lines.append(add_import)
         self.number_of_lines = len(self.in_lines)
 
+        if not extension:
+            self.extension = file_name.split('.')[-1] if file_name else "py"
+        else:
+            self.extension = extension
+
         self.out_lines = []
         self.comments = {'from': {}, 'straight': {}, 'nested': {}, 'above': {'straight': {}, 'from': {}}}
         self.imports = OrderedDict()
@@ -236,6 +241,8 @@ class SortImports(object):
         if show_diff or self.config['show_diff']:
             self._show_diff(file_contents)
         elif write_to_stdout:
+            if sys.version_info[0] < 3:
+                self.output = self.output.encode(self.file_encoding)
             sys.stdout.write(self.output)
         elif file_name and not check:
             if self.output == file_contents:
@@ -670,8 +677,10 @@ class SortImports(object):
 
             if self.config['lines_after_imports'] != -1:
                 self.out_lines[imports_tail:0] = ["" for line in range(self.config['lines_after_imports'])]
-            elif next_construct.startswith("def ") or next_construct.startswith("class ") or \
-               next_construct.startswith("@") or next_construct.startswith("async def"):
+            elif self.extension != "pyi" and (next_construct.startswith("def ") or
+                                              next_construct.startswith("class ") or
+                                              next_construct.startswith("@") or
+                                              next_construct.startswith("async def")):
                 self.out_lines[imports_tail:0] = ["", ""]
             else:
                 self.out_lines[imports_tail:0] = [""]
@@ -829,7 +838,7 @@ class SortImports(object):
             self._in_top_comment = True
             return True
         elif self._in_top_comment:
-            if not line.startswith("#"):
+            if not line.startswith("#") or line in self._section_comments:
                 self._in_top_comment = False
                 self._first_comment_index_end = self.index - 1
 
